@@ -1,6 +1,6 @@
 /* gcompris - algebra.c
  *
- * Time-stamp: <2001/09/12 23:44:41 bruno>
+ * Time-stamp: <2002/04/17 00:25:30 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -35,7 +35,13 @@
 
 #define SOUNDLISTFILE PACKAGE
 
-static GList *item_button_list = NULL;
+gboolean	 board_paused = TRUE;
+static int	 leavenow;
+static int	 gamewon;
+static void	 game_won(void);
+
+static int	 maxtime;
+
 static GList *item_list = NULL;
 
 static GcomprisBoard *gcomprisBoard = NULL;
@@ -60,32 +66,31 @@ typedef struct {
 } ToBeFoundItem;
 static ToBeFoundItem *currentToBeFoundItem = NULL;
 
+static GnomeCanvasGroup *boardRootItem = NULL;
+
 static char currentOperation = PLUSSIGNFILE;
 
-static void start_board (GcomprisBoard *agcomprisBoard);
-static void pause_board (gboolean pause);
-static void end_board (void);
-static gboolean is_our_board (GcomprisBoard *gcomprisBoard);
-static void set_level (guint level);
-gint key_press(guint keyval);
-static void process_ok(void);
+static void		 start_board (GcomprisBoard *agcomprisBoard);
+static void		 pause_board (gboolean pause);
+static void		 end_board (void);
+static gboolean		 is_our_board (GcomprisBoard *gcomprisBoard);
+static void		 set_level (guint level);
+gint			 key_press(guint keyval);
+static void		 process_ok(void);
 
-static void algebra_create_buttons(GnomeCanvasGroup *parent);
-static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent);
-static void algebra_destroy_item(GnomeCanvasItem *item);
-static void algebra_destroy_all_items(void);
-static void display_operand(GnomeCanvasGroup *parent, 
-			    double x_align,
-			    double y, 
-			    char *operand_str,
-			    gboolean masked);
-static void get_random_number(guint *first_operand, guint *second_operand);
-static void algebra_next_level(void);
-static gint item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-static gint item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-static gint item_event_horver(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
-static void set_focus_item(ToBeFoundItem *toBeFoundItem, gboolean status);
-static void init_operation();
+static GnomeCanvasItem	*algebra_create_item(GnomeCanvasGroup *parent);
+static void		 algebra_destroy_item(GnomeCanvasItem *item);
+static void		 algebra_destroy_all_items(void);
+static void		 display_operand(GnomeCanvasGroup *parent, 
+					 double x_align,
+					 double y, 
+					 char *operand_str,
+					 gboolean masked);
+static void		 get_random_number(guint *first_operand, guint *second_operand);
+static void		 algebra_next_level(void);
+static gint		 item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data);
+static void		 set_focus_item(ToBeFoundItem *toBeFoundItem, gboolean status);
+static void		 init_operation(void);
 
 /* Description of this plugin */
 BoardPlugin menu_bp =
@@ -106,6 +111,8 @@ BoardPlugin menu_bp =
    key_press,
    process_ok,
    set_level,
+   NULL,
+   NULL
 };
 
 /*
@@ -126,10 +133,18 @@ BoardPlugin
  */
 static void pause_board (gboolean pause)
 {
-
   if(gcomprisBoard==NULL)
     return;
 
+  if(gamewon == TRUE && pause == FALSE) /* the game is won */
+    {
+      algebra_next_level();
+    }
+
+  if(leavenow == TRUE && pause == FALSE)
+    board_finished(BOARD_FINISHED_RANDOM);
+
+  board_paused = pause;
 }
 
 /*
@@ -146,15 +161,39 @@ static void start_board (GcomprisBoard *agcomprisBoard)
       /* set initial values for this level */
       gcomprisBoard->level=1;
       gcomprisBoard->maxlevel=9;
-      gcomprisBoard->sublevel=0;
+      gcomprisBoard->sublevel=1;
       gcomprisBoard->number_of_sublevel=10; /* Go to next level after this number of 'play' */
+      gcompris_score_start(SCORESTYLE_NOTE, 
+			   gcomprisBoard->width - 220, 
+			   gcomprisBoard->height - 50, 
+			   gcomprisBoard->number_of_sublevel);
       gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_OK);
 
+      /* The mode defines the operation */
+      /* If added with the sign | then the vertical layout is selected */
+
+      /* Default mode */
+      if(!gcomprisBoard->mode)
+	currentOperation=PLUSSIGNFILE;
+      else if(g_strncasecmp(gcomprisBoard->mode, "+", 1)==0)
+	currentOperation=PLUSSIGNFILE;
+      else if(g_strncasecmp(gcomprisBoard->mode, "-", 1)==0)
+	currentOperation=MINUSSIGNFILE;
+      else if(g_strncasecmp(gcomprisBoard->mode, "*", 1)==0)
+	currentOperation=BYSIGNFILE;
+      else if(g_strncasecmp(gcomprisBoard->mode, "/", 1)==0)
+	currentOperation=DIVIDESIGNFILE;
+      
+      if(g_strncasecmp(gcomprisBoard->mode+1, "|", 1)==0)
+	 vertical_layout=TRUE;
+      else
+	vertical_layout=FALSE;
+
       init_operation();
       algebra_next_level();
 
-      algebra_create_buttons(gnome_canvas_root(gcomprisBoard->canvas));
-
+      gamewon = FALSE;
+      leavenow = FALSE;
       pause_board(FALSE);
     }
 
@@ -163,13 +202,13 @@ static void start_board (GcomprisBoard *agcomprisBoard)
 static void
 end_board ()
 {
-
-
   if(gcomprisBoard!=NULL)
     {
       pause_board(TRUE);
+      gcompris_score_end();
       algebra_destroy_all_items();
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -180,7 +219,7 @@ set_level (guint level)
   if(gcomprisBoard!=NULL)
     {
       gcomprisBoard->level=level;
-      gcomprisBoard->sublevel=0;
+      gcomprisBoard->sublevel=1;
       algebra_next_level();
     }
 }
@@ -190,8 +229,8 @@ gint key_press(guint keyval)
   guint c;
   gboolean stop = FALSE;
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -344,19 +383,36 @@ is_our_board (GcomprisBoard *gcomprisBoard)
 /*-------------------------------------------------------------------------------*/
 /*-------------------------------------------------------------------------------*/
 
+static void timer_end()
+{
+  gamewon = FALSE;
+  leavenow = TRUE;
+  algebra_destroy_all_items();
+  gcompris_display_bonus(gamewon, BONUS_SMILEY);
+}
+
+
 /* set initial values for the next level */
 static void algebra_next_level() 
 {
 
   gcompris_bar_set_level(gcomprisBoard);
-
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
+  gcompris_score_set(gcomprisBoard->sublevel);
 
   algebra_destroy_all_items();
 
+  boardRootItem = GNOME_CANVAS_GROUP(
+				     gnome_canvas_item_new (gnome_canvas_root(gcomprisBoard->canvas),
+							    gnome_canvas_group_get_type (),
+							    "x", (double) 0,
+							    "y", (double) 0,
+							    NULL));
+
+  maxtime = 20;
+  gcompris_timer_display(150, 140, GCOMPRIS_TIMER_BALLOON, maxtime, timer_end);
+
   /* Try the next level */
-  algebra_create_item(gnome_canvas_root(gcomprisBoard->canvas));
+  algebra_create_item(boardRootItem);
 
 }
 
@@ -373,6 +429,8 @@ static void algebra_destroy_all_items()
   GnomeCanvasItem *item;
   gboolean stop = FALSE;
 
+  gcompris_timer_end();
+
   while(g_list_length(item_list)>0) 
     {
       item = g_list_nth_data(item_list, 0);
@@ -402,6 +460,11 @@ static void algebra_destroy_all_items()
 	    stop = TRUE;
 	}
     }
+
+  if(boardRootItem!=NULL)
+    gtk_object_destroy (GTK_OBJECT(boardRootItem));
+  
+  boardRootItem=NULL;
 }
 
 static void display_operand(GnomeCanvasGroup *parent, 
@@ -513,238 +576,6 @@ static void display_operand(GnomeCanvasGroup *parent,
     }
 }
 
-static void algebra_create_buttons(GnomeCanvasGroup *parent)
-{
-  GnomeCanvasItem *item;
-  GdkPixbuf *algebra_pixmap = NULL;
-  GdkPixbuf *button_pixmap = NULL;
-  double y_offset;
-  double x_offset = 20;
-  static char plus_operation[1]; 
-  static char minus_operation[1];
-  static char by_operation[1];
-  static char divide_operation[1];
-
-  plus_operation[0]   = PLUSSIGNFILE;
-  minus_operation[0]  = MINUSSIGNFILE;
-  by_operation[0]     = BYSIGNFILE;
-  divide_operation[0] = DIVIDESIGNFILE;
-
-  /*----- PLUS -----*/
-  button_pixmap = gcompris_load_pixmap("gcompris/buttons/button.png");
-  y_offset = (double)gdk_pixbuf_get_height(button_pixmap) + 20;
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
-				"x", x_offset,
-				"y", (double) gcomprisBoard->height/4,
-				"width", (double)  gdk_pixbuf_get_width(button_pixmap),
-				"height", (double) gdk_pixbuf_get_height(button_pixmap),
-				NULL);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &plus_operation);
-  gcompris_set_image_focus(button_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     button_pixmap);
-
-  algebra_pixmap = gcompris_load_operation_pixmap(PLUSSIGNFILE);
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", (double) x_offset + (gdk_pixbuf_get_width(button_pixmap) 
-							  - gdk_pixbuf_get_width(algebra_pixmap))/2,
-				"y", (double) gcomprisBoard->height/4 + 
-				(gdk_pixbuf_get_height(button_pixmap) 
-				 -  gdk_pixbuf_get_height(algebra_pixmap))/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				NULL);
-  gdk_pixbuf_unref(algebra_pixmap);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &plus_operation);
-  gcompris_set_image_focus(algebra_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     algebra_pixmap);
-
-
-  /*----- MINUS -----*/
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
-				"x", x_offset,
-				"y", (double) gcomprisBoard->height/4 + y_offset,
-				"width", (double)  gdk_pixbuf_get_width(button_pixmap),
-				"height", (double) gdk_pixbuf_get_height(button_pixmap),
-				NULL);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &minus_operation);
-  gcompris_set_image_focus(button_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     button_pixmap);
-
-  algebra_pixmap = gcompris_load_operation_pixmap(MINUSSIGNFILE);
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", (double) x_offset + (gdk_pixbuf_get_width(button_pixmap) 
-							  - gdk_pixbuf_get_width(algebra_pixmap))/2,
-				"y", (double) gcomprisBoard->height/4 + y_offset +
-				(gdk_pixbuf_get_height(button_pixmap) -  gdk_pixbuf_get_height(algebra_pixmap))/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				NULL);
-  gdk_pixbuf_unref(algebra_pixmap);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &minus_operation);
-  gcompris_set_image_focus(algebra_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     algebra_pixmap);
-
-  /*----- BY -----*/
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
-				"x", x_offset,
-				"y", (double) gcomprisBoard->height/4 + y_offset*2,
-				"width", (double)  gdk_pixbuf_get_width(button_pixmap),
-				"height", (double) gdk_pixbuf_get_height(button_pixmap),
-				NULL);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &by_operation);
-  gcompris_set_image_focus(button_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     button_pixmap);
-
-  algebra_pixmap = gcompris_load_operation_pixmap(BYSIGNFILE);
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", (double) x_offset + (gdk_pixbuf_get_width(button_pixmap) - 
-							  gdk_pixbuf_get_width(algebra_pixmap))/2,
-				"y", (double) gcomprisBoard->height/4 + y_offset*2 +
-				(gdk_pixbuf_get_height(button_pixmap) -  gdk_pixbuf_get_height(algebra_pixmap))/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				NULL);
-  gdk_pixbuf_unref(algebra_pixmap);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &by_operation);
-  gcompris_set_image_focus(algebra_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     algebra_pixmap);
-
-
-  /*----- DIVIDE -----*/
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
-				"x", x_offset,
-				"y", (double) gcomprisBoard->height/4 + y_offset*3,
-				"width", (double)  gdk_pixbuf_get_width(button_pixmap),
-				"height", (double) gdk_pixbuf_get_height(button_pixmap),
-				NULL);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &divide_operation);
-  gcompris_set_image_focus(button_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     button_pixmap);
-
-  algebra_pixmap = gcompris_load_operation_pixmap(DIVIDESIGNFILE); 
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", (double) x_offset + (gdk_pixbuf_get_width(button_pixmap) 
-							  - gdk_pixbuf_get_width(algebra_pixmap))/2,
-				"y", (double) gcomprisBoard->height/4 + y_offset*3 +
-				(gdk_pixbuf_get_height(button_pixmap) -  gdk_pixbuf_get_height(algebra_pixmap))/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				NULL);
-  gdk_pixbuf_unref(algebra_pixmap);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_valid,
-		     &divide_operation);
-  gcompris_set_image_focus(algebra_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     algebra_pixmap);
-
-  /*----- HORIZONTAL / VERTICAL LAYOUT SELECTION -----*/
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf",  button_pixmap, 
-				"x", x_offset,
-				"y", (double) gcomprisBoard->height/4 + y_offset*4 + 20,
-				"width", (double)  gdk_pixbuf_get_width(button_pixmap),
-				"height", (double) gdk_pixbuf_get_height(button_pixmap),
-				NULL);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_horver,
-		     NULL);
-  gcompris_set_image_focus(button_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     button_pixmap);
-
-
-  algebra_pixmap = gcompris_load_pixmap("gcompris/buttons/horver.png");
-  item = gnome_canvas_item_new (parent,
-				gnome_canvas_pixbuf_get_type (),
-				"pixbuf", algebra_pixmap, 
-				"x", (double) x_offset + (gdk_pixbuf_get_width(button_pixmap) 
-							  - gdk_pixbuf_get_width(algebra_pixmap))/2,
-				"y", (double) gcomprisBoard->height/4 + y_offset*4 + 20 +
-				(gdk_pixbuf_get_height(button_pixmap) -  gdk_pixbuf_get_height(algebra_pixmap))/2,
-				"width", (double)  gdk_pixbuf_get_width(algebra_pixmap),
-				"height", (double) gdk_pixbuf_get_height(algebra_pixmap),
-				NULL);
-  gdk_pixbuf_unref(algebra_pixmap);
-  item_button_list = g_list_append (item_button_list, item);
-
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) item_event_horver,
-		     &divide_operation);
-  gcompris_set_image_focus(algebra_pixmap, FALSE);
-  gtk_signal_connect(GTK_OBJECT(item), "event",
-		     (GtkSignalFunc) gcompris_item_event_focus,
-		     algebra_pixmap);
-
-  /* Cleanup */
-  gdk_pixbuf_unref(button_pixmap);
-}
-
 static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
 {
   GdkPixbuf *algebra_pixmap = NULL;
@@ -752,6 +583,7 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
   guint first_operand, second_operand;
   char *first_operand_str = NULL;
   char *second_operand_str = NULL;
+  char *audioOperand = NULL;
   guint longuest;
   double x_align;
   double x, y;
@@ -773,9 +605,9 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
 
   if(vertical_layout)
     /* Vertical layout : Warning x_align is the right assigned value for display_operand */
-    x_align = gcomprisBoard->width - (gcomprisBoard->width - (longuest*NUMBERSWIDTH))/2;
+    x_align = gcomprisBoard->width - (gcomprisBoard->width - (longuest*NUMBERSWIDTH))/2 + 150;
   else
-    x_align = (gcomprisBoard->width - (longuest*3*NUMBERSWIDTH))/2 + NUMBERSWIDTH*(strlen(first_operand_str));
+    x_align = (gcomprisBoard->width - (longuest*3*NUMBERSWIDTH))/2 + NUMBERSWIDTH*(strlen(first_operand_str)) + 150;
 
   /* First operand */
   display_operand(parent, x_align, y_firstline, first_operand_str, FALSE);
@@ -844,15 +676,19 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
     {
     case PLUSSIGNFILE:
       expected_result = g_strdup_printf("%d", first_operand+second_operand);
+      audioOperand = "plus";
       break;
     case MINUSSIGNFILE:
       expected_result = g_strdup_printf("%d", first_operand-second_operand);
+      audioOperand = "minus";
       break;
     case BYSIGNFILE:
       expected_result = g_strdup_printf("%d", first_operand*second_operand);
+      audioOperand = "by";
       break;
     case DIVIDESIGNFILE:
       expected_result = g_strdup_printf("%d", first_operand/second_operand);
+      audioOperand = "outof";
       break;
     default:
       g_error("Bad Operation");
@@ -864,6 +700,9 @@ static GnomeCanvasItem *algebra_create_item(GnomeCanvasGroup *parent)
 						    strlen(expected_result)+2),
 		    y_firstline, expected_result, TRUE);
 
+
+  gcompris_play_ogg(first_operand_str, audioOperand , second_operand_str, "equal", NULL);
+
   g_free(first_operand_str);
   g_free(second_operand_str);
 
@@ -920,23 +759,7 @@ static void process_ok()
   
   if(hasfail==NULL)
     {
-      gcomprisBoard->sublevel++;
-      gcompris_bar_set_timer(gcomprisBoard->sublevel);
-
-      if(gcomprisBoard->sublevel>=gcomprisBoard->number_of_sublevel) {
-	/* Try the next level */
-	gcomprisBoard->level++;
-	if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-	  gcomprisBoard->level=gcomprisBoard->maxlevel;
-
-	gcomprisBoard->sublevel=0;
-	init_operation();
-	gcompris_play_sound (SOUNDLISTFILE, "bonus");
-      } else {
-	gcompris_play_sound (SOUNDLISTFILE, "gobble");
-      }
-
-      algebra_next_level();
+      game_won();
     }
   else
     {
@@ -945,61 +768,6 @@ static void process_ok()
     }
 }
 
-/* Callback for the ok and operation buttons */
-static gint
-item_event_valid(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
-{
-
-  switch (event->type) 
-    {
-    case GDK_ENTER_NOTIFY:
-      break;
-    case GDK_LEAVE_NOTIFY:
-      break;
-    case GDK_BUTTON_PRESS:
-      if(currentOperation==((char *)data)[0])
-	{
-	  return FALSE;
-	}
-      else 
-	{
-	  currentOperation=((char *)data)[0];
-	  init_operation();
-	  algebra_next_level();
-	}
-      gcompris_play_sound (SOUNDLISTFILE, "gobble");
-      break;
-      
-    default:
-      break;
-    }
-  return FALSE;
-
-}
-
-/* Callback for the horizontal / vertical layout change */
-static gint item_event_horver(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
-{
-  switch (event->type) 
-    {
-    case GDK_BUTTON_PRESS:
-      if(vertical_layout==TRUE)
-	vertical_layout=FALSE;
-      else
-	vertical_layout=TRUE;
-
-      algebra_next_level();
-
-      gcompris_play_sound (SOUNDLISTFILE, "gobble");
-      break;
-      
-    default:
-      break;
-    }
-  return FALSE;
-}
-
-
 /* Callback for the 'toBeFoundItem' */
 static gint
 item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
@@ -1008,6 +776,9 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, gpointer data)
 
   toBeFoundItem=(ToBeFoundItem *)data;
 
+  if(board_paused)
+    return FALSE;
+
   switch (event->type) 
     {
     case GDK_ENTER_NOTIFY:
@@ -1071,44 +842,16 @@ static void get_random_number(guint *first_operand, guint *second_operand)
   switch(currentOperation)
     {
     case PLUSSIGNFILE:
+      *first_operand = get_operand();
+      *second_operand  = gcomprisBoard->level;
+      break;
     case BYSIGNFILE:
       *first_operand  = gcomprisBoard->level;
       *second_operand = get_operand();
       break;
     case MINUSSIGNFILE:
-      switch(gcomprisBoard->level)
-	{
-	case 1 :
-	  min = 0;
-	  max = 2;
-	  break;
-	case 2 :
-	  min = 2;
-	  max = 3;
-	  break;
-	case 3 :
-	  min = 4;
-	  max = 5;
-	  break;
-	case 4 :
-	  min = 6;
-	  max = 7;
-	  break;
-	case 5 :
-	  min = 8;
-	  max = 9;
-	  break;
-	case 6 :
-	  min = 1;
-	  max = 10;
-	  break;
-	default :
-	  min = 1;
-	  max = 10;
-	}
-      *second_operand = (min+rand()%(max-min+1));
-      *first_operand  = MAX((min+rand()%(20-min)), *second_operand);
-      *first_operand  = MAX((min+rand()%(20-min)), *second_operand);
+      *first_operand  = gcomprisBoard->level+9;
+      *second_operand = get_operand();
       break;
     case DIVIDESIGNFILE:
       switch(gcomprisBoard->level)
@@ -1149,6 +892,29 @@ static void get_random_number(guint *first_operand, guint *second_operand)
     }
 }
 
+/* ==================================== */
+static void game_won()
+{
+  gcomprisBoard->sublevel++;
+
+  if(gcomprisBoard->sublevel>gcomprisBoard->number_of_sublevel) {
+    /* Try the next level */
+    gcomprisBoard->sublevel=1;
+    gcomprisBoard->level++;
+    if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+      board_finished(BOARD_FINISHED_RANDOM);
+      return;
+    }
+
+    gamewon = TRUE;
+    algebra_destroy_all_items();
+    gcompris_display_bonus(gamewon, BONUS_SMILEY);
+    init_operation();
+    return;
+  }
+  algebra_next_level();
+}
+
 
 
 
