@@ -1,6 +1,6 @@
 /* gcompris - paratrooper.c
  *
- * Time-stamp: <2001/09/12 23:41:48 bruno>
+ * Time-stamp: <2002/04/14 00:25:03 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -23,13 +23,14 @@
 
 #define SOUNDLISTFILE PACKAGE
 
+static int gamewon;
+
 static GList *item_list = NULL;
 static GList *item2del_list = NULL;
 
 static GcomprisBoard *gcomprisBoard = NULL;
 
 static gint dummy_id = 0;
-static gint drop_cloud_id = 0;
 static gint drop_tux_id = 0;
 
 static GnomeCanvasItem *seaitem = NULL;
@@ -73,22 +74,24 @@ static  ParatrooperItem	     paratrooperItem;
 /* default gnome pixmap directory in which this game tales the icon */
 static char *pixmapsdir = "gcompris/misc/";
 
-static void	 start_board (GcomprisBoard *agcomprisBoard);
-static void	 pause_board (gboolean pause);
-static void	 end_board (void);
-static gboolean	 is_our_board (GcomprisBoard *gcomprisBoard);
-static void	 set_level (guint level);
-gint		 key_press(guint keyval);
-static void	 help(void);
+static void	 	 start_board (GcomprisBoard *agcomprisBoard);
+static void	 	 pause_board (gboolean pause);
+static void	 	 end_board (void);
+static gboolean	 	 is_our_board (GcomprisBoard *gcomprisBoard);
+static void	 	 set_level (guint level);
+gint		 	 key_press(guint keyval);
+
+static GnomeCanvasItem	*paratrooper_create_cloud(GnomeCanvasGroup *parent);
+static gint		 paratrooper_drop_clouds (GtkWidget *widget, gpointer data);
+static gint		 paratrooper_move_items (GtkWidget *widget, gpointer data);
+static gint		 paratrooper_move_tux (GtkWidget *widget, gpointer data);
+static void		 paratrooper_destroy_item(CloudItem *clouditem);
+static void		 paratrooper_destroy_items(void);
+static void		 paratrooper_destroy_all_items(void);
+static void		 paratrooper_next_level(void);
+static gint		 item_event(GnomeCanvasItem *item, GdkEvent *event, void *data);
+static void		 next_state(void);
 
-static GnomeCanvasItem *paratrooper_create_cloud(GnomeCanvasGroup *parent);
-static gint paratrooper_drop_clouds (GtkWidget *widget, gpointer data);
-static gint paratrooper_move_items (GtkWidget *widget, gpointer data);
-static gint paratrooper_move_tux (GtkWidget *widget, gpointer data);
-static void paratrooper_destroy_item(CloudItem *clouditem);
-static void paratrooper_destroy_items(void);
-static void paratrooper_destroy_all_items(void);
-static void paratrooper_next_level(void);
 
 static  double               speed = 0.0;
 static  double               imageZoom = 0.0;
@@ -112,7 +115,8 @@ BoardPlugin menu_bp =
    key_press,
    NULL,
    set_level,
-   help,
+   NULL,
+   NULL
 };
 
 /*
@@ -142,10 +146,6 @@ static void pause_board (gboolean pause)
 	gtk_timeout_remove (dummy_id);
 	dummy_id = 0;
       }
-      if (drop_cloud_id) {
-	gtk_timeout_remove (drop_cloud_id);
-	drop_cloud_id = 0;
-      }
       if (drop_tux_id) {
 	gtk_timeout_remove (drop_tux_id);
 	drop_tux_id = 0;
@@ -153,16 +153,26 @@ static void pause_board (gboolean pause)
     }
   else
     {
-      if(!drop_cloud_id) {
-	drop_cloud_id = gtk_timeout_add (1000,
-					 (GtkFunction) paratrooper_drop_clouds, NULL);
-      }
+      if(gamewon == TRUE) /* the game is won */
+	{
+	  gcomprisBoard->level++;
+	  if(gcomprisBoard->level>gcomprisBoard->maxlevel) { // the current board is finished : bail out
+	    board_finished(BOARD_FINISHED_RANDOM);
+	    return;
+	  }
+	  printf("paratrooper pause start next level\n");
+	}
+
+      // Unpause code
       if(!dummy_id) {
 	dummy_id = gtk_timeout_add (1000, (GtkFunction) paratrooper_move_items, NULL);
       }
       if(paratrooperItem.status!=TUX_INPLANE && paratrooperItem.status!=TUX_LANDED) {
 	drop_tux_id = gtk_timeout_add (1000, (GtkFunction) paratrooper_move_tux, NULL);
       }
+
+      if(gamewon == TRUE) /* the game is won */
+	  paratrooper_next_level();
     }
 }
 
@@ -181,7 +191,7 @@ static void start_board (GcomprisBoard *agcomprisBoard)
       /* set initial values for this level */
       gcomprisBoard->level = 1;
       gcomprisBoard->maxlevel = 6;
-      gcompris_bar_set(GCOMPRIS_BAR_LEVEL|GCOMPRIS_BAR_HELP);
+      gcompris_bar_set(GCOMPRIS_BAR_LEVEL);
 
       /* Init of paratrooper struct */
       paratrooperItem.rootitem = NULL;
@@ -204,6 +214,7 @@ end_board ()
       paratrooper_destroy_all_items();
       gcomprisBoard->level = 1;       // Restart this game to zero
     }
+  gcomprisBoard = NULL;
 }
 
 static void
@@ -220,8 +231,9 @@ set_level (guint level)
 gint key_press(guint keyval)
 {
 
-   if(!get_board_playing())
-     return FALSE;
+  if(!gcomprisBoard)
+    return TRUE;
+
 
   /* Add some filter for control and shift key */
   switch (keyval)
@@ -291,49 +303,16 @@ gint key_press(guint keyval)
     case GDK_Left:
       break;
     case GDK_Up:
-      if(paratrooperItem.speed > 3)
+      if(paratrooperItem.speed >= 3)
 	paratrooperItem.speed--;
       return TRUE;
     case GDK_Down:
-      if(paratrooperItem.speed < 6)
+      if(paratrooperItem.speed <= 6)
       paratrooperItem.speed++;
       return TRUE;
     }
 
-  switch(paratrooperItem.status)
-    {
-    case TUX_INPLANE:
-      gnome_canvas_item_move(paratrooperItem.rootitem, plane_x+100, 0);
-      paratrooperItem.x += plane_x+100;
-      gnome_canvas_item_show(paratrooperItem.paratrooper);
-      paratrooperItem.status = TUX_DROPPING;
-      drop_tux_id = gtk_timeout_add (10, (GtkFunction) paratrooper_move_tux, NULL);
-      break;
-    case TUX_DROPPING:
-      gnome_canvas_item_lower_to_bottom(paratrooperItem.parachute);
-      gnome_canvas_item_show(paratrooperItem.parachute);
-      paratrooperItem.speed /= 3;
-      paratrooperItem.status = TUX_FLYING;
-      break;
-    case TUX_LANDED:
-      gcomprisBoard->level++;
-      if(gcomprisBoard->level>gcomprisBoard->maxlevel)
-	gcomprisBoard->level=gcomprisBoard->maxlevel;
-      paratrooper_next_level();
-      break;
-    case TUX_CRASHED:
-      /* Restart */
-      gnome_canvas_item_move(paratrooperItem.rootitem, -paratrooperItem.x, -paratrooperItem.y+60);
-      paratrooperItem.status	= TUX_INPLANE;
-      paratrooperItem.x		= 0;
-      paratrooperItem.y		= 60;
-      paratrooperItem.speed	= 3;
-      gnome_canvas_item_hide(paratrooperItem.paratrooper);
-      gnome_canvas_item_show(planeitem);
-      break;
-    default:
-      break;
-    }
+  next_state();
 
   return TRUE;
 }
@@ -354,26 +333,6 @@ is_our_board (GcomprisBoard *gcomprisBoard)
   return FALSE;
 }
 
-static void
-help ()
-{
-  if(gcomprisBoard!=NULL)
-    {
-      pause_board(TRUE);
-      gcompris_help_start(N_("The paratrooper game"), 
-			  N_("\
-Goal:\n\
-In this game, the children will try to make Tux the paratrooper lands safely on the\n\
-fishing boat. He/She needs to take care of the wind side and speed.\n\
-\n\
-Usage:\n\
-Click on any keyboard key to let Tux jump. Another keypress will open the
-'parachute'.\n
-"));
-			  
-    }
-}
-
 /*-------------------------------------------------------------------------------*/
 /*-------------------------------------------------------------------------------*/
 /*-------------------------------------------------------------------------------*/
@@ -385,18 +344,17 @@ static void paratrooper_next_level()
   GdkPixbuf *pixmap = NULL;
   char *str = NULL;
 
+  gamewon = FALSE;
+
   gcompris_bar_set_level(gcomprisBoard);
 
   paratrooper_destroy_all_items();
 
   /* Try the next level */
   speed=100+(30/(gcomprisBoard->level));
-  gcomprisBoard->number_of_sublevel=1;
+  gcomprisBoard->number_of_sublevel=0;
   /* Make the images tend to 0.5 ratio */
   imageZoom=0.3+(0.4/(gcomprisBoard->level));
-  gcompris_bar_set_maxtimer(gcomprisBoard->number_of_sublevel);
-  gcomprisBoard->sublevel=0;
-  gcompris_bar_set_timer(gcomprisBoard->sublevel);
 
   /* Setup and Display the plane */
   planespeed_y = 0;
@@ -415,25 +373,25 @@ static void paratrooper_next_level()
 				     "width_set", TRUE, 
 				     "height_set", TRUE,
 				     NULL);
+  gtk_signal_connect(GTK_OBJECT(planeitem), "event",
+		     (GtkSignalFunc) item_event,
+		     NULL);
   gdk_pixbuf_unref(pixmap);
  
   windspeed = (3 + rand()%gcomprisBoard->level);
   if(rand()%2==0)
     windspeed *= -1;
 
-  /* Reset the cloud timer to be sure to launch a new cloud asap */
-  if(drop_cloud_id)
-    gtk_timeout_remove (drop_cloud_id);
-
-  drop_cloud_id = gtk_timeout_add (200,
-				   (GtkFunction) paratrooper_drop_clouds, NULL);
+  /* Drop a cloud */
+  gtk_timeout_add (200,
+		   (GtkFunction) paratrooper_drop_clouds, NULL);
 
   /* Display the target */
   g_free(str);
   str = g_strdup_printf("%s%s", pixmapsdir, "fishingboat.png");
   pixmap = gcompris_load_pixmap(str);
   boat_x = 350;
-  boat_y = 500;
+  boat_y = gcomprisBoard->height-100;
   boat_landarea_y = boat_y+20;
   boat_length = gdk_pixbuf_get_width(pixmap)*imageZoom;
   boatitem = gnome_canvas_item_new (gnome_canvas_root(gcomprisBoard->canvas),
@@ -494,6 +452,9 @@ static void paratrooper_next_level()
 						       NULL);
   gnome_canvas_item_hide(paratrooperItem.paratrooper);
   gdk_pixbuf_unref(pixmap);
+  gtk_signal_connect(GTK_OBJECT(paratrooperItem.paratrooper), "event",
+		     (GtkSignalFunc) item_event,
+		     NULL);
 
   g_free(str);
   str = g_strdup_printf("%s%s", pixmapsdir, "parachute.png");
@@ -513,10 +474,6 @@ static void paratrooper_next_level()
   /* The sea is always on top */
   gnome_canvas_item_raise_to_top(seaitem);
 
-  /* status bar */
-  gcompris_bar_set_maxtimer(1);
-  gcompris_bar_set_timer(0);
-
   g_free (str);
 
 }
@@ -558,16 +515,16 @@ static void paratrooper_move_cloud(CloudItem *clouditem)
 				   &y1,
 				   &x2,
 				   &y2);
-  
-  if((windspeed<0 && x2<0) || (windspeed>0 && x1>gcomprisBoard->width))
+
+  /* Manage the wrapping for the cloud */  
+  if(windspeed<0 && x2<0)
     {
-      item2del_list = g_list_append (item2del_list, clouditem);
-
-      /* Drop a new cloud */
-      drop_cloud_id = gtk_timeout_add (200,
-				       (GtkFunction) paratrooper_drop_clouds, NULL);
+      gnome_canvas_item_move(item, gcomprisBoard->width, 0.0);
+    }
+  else if(windspeed>0 && x1>gcomprisBoard->width)
+    {
+      gnome_canvas_item_move(item, -gcomprisBoard->width, 0.0);
     }
-
 }
 
 static void paratrooper_destroy_item(CloudItem *clouditem)
@@ -671,7 +628,6 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
   gnome_canvas_item_move(paratrooperItem.rootitem, offset, paratrooperItem.speed);
   paratrooperItem.y += paratrooperItem.speed;
   paratrooperItem.x += offset;
-  printf("paratrooperItem.x = %d\n", paratrooperItem.x);
 
   /* Check we reached the target */
   if(paratrooperItem.y>boat_landarea_y)
@@ -682,6 +638,7 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
 	  gnome_canvas_item_hide(paratrooperItem.parachute);
 	  paratrooperItem.status = TUX_LANDED;
 	  gcompris_play_sound (SOUNDLISTFILE, "bonus");
+	  next_state();
 	}
       else
 	{
@@ -693,6 +650,7 @@ static gint paratrooper_move_tux (GtkWidget *widget, gpointer data)
 	      gnome_canvas_item_hide(paratrooperItem.parachute);
 	      paratrooperItem.status = TUX_CRASHED;
 	      gcompris_play_sound (SOUNDLISTFILE, "crash");
+	      next_state();
 	    }
 	}
     }
@@ -775,6 +733,69 @@ static gint paratrooper_drop_clouds (GtkWidget *widget, gpointer data)
 }
 
 
+
+/*
+ * This is the state machine of the paratrooper
+ */
+void next_state()
+{
+
+  switch(paratrooperItem.status)
+    {
+    case TUX_INPLANE:
+      gnome_canvas_item_move(paratrooperItem.rootitem, plane_x+100, 0);
+      paratrooperItem.x += plane_x+100;
+      gnome_canvas_item_show(paratrooperItem.paratrooper);
+      paratrooperItem.status = TUX_DROPPING;
+      drop_tux_id = gtk_timeout_add (10, (GtkFunction) paratrooper_move_tux, NULL);
+      break;
+    case TUX_DROPPING:
+      gnome_canvas_item_lower_to_bottom(paratrooperItem.parachute);
+      gnome_canvas_item_show(paratrooperItem.parachute);
+      paratrooperItem.speed /= 3;
+      paratrooperItem.status = TUX_FLYING;
+      break;
+    case TUX_LANDED:
+      gamewon = TRUE;
+      gcompris_display_bonus(gamewon, BONUS_RANDOM);
+      break;
+    case TUX_CRASHED:
+      /* Restart */
+      gnome_canvas_item_move(paratrooperItem.rootitem, -paratrooperItem.x, -paratrooperItem.y+60);
+      paratrooperItem.status	= TUX_INPLANE;
+      paratrooperItem.x		= 0;
+      paratrooperItem.y		= 60;
+      paratrooperItem.speed	= 3;
+      gnome_canvas_item_hide(paratrooperItem.paratrooper);
+      gnome_canvas_item_show(planeitem);
+      break;
+    default:
+      break;
+    }
+}
+
+static gint
+item_event(GnomeCanvasItem *item, GdkEvent *event, void *data)
+{
+
+  if(!gcomprisBoard)
+    return FALSE;
+
+   switch (event->type) 
+     {
+     case GDK_BUTTON_PRESS:
+       next_state();
+       break;
+
+     default:
+       break;
+     }
+         
+   return FALSE;
+ }
+
+
+
 
 /* Local Variables: */
 /* mode:c */
